from f1_4667 import F1Res4667
from f2_4667 import F2Res4667
from f3_4667 import F3Res4667
from f4_4667 import F4Res4667
from f5_4667 import F5Res4667
from f6_4667 import F6Res4667
from f7_4667 import F7Res4667
from f8_4667 import F8Res4667
from macro_4667 import MacroRes4667
from resumen_4667 import ResumenRes4667
from resumen_ccaa_4667 import ResumenCCAARes4667
from pro_4667 import ProyectoRes4667
